#include<iostream.h>
#include"defines.h"

bool ShowLicense() {
  cout << "Achilles " << ACHILLES_VERSION << ", Copyright (C) 2000 Matthew Danish" << endl;
  cout << "Achilles comes with ABSOLUTELY NO WARRANTY; for details" << endl;
  cout << "read the included file 'COPYING'.  This is free software, and" << endl;
  cout << "you are welcome to redistribute it under certain conditions," << endl;
  cout << "read the file 'COPYING' for information." << endl;
  return true;
}


