/* 

Copyright (C) 2000 Matthew Danish

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef GENE_H_4378354
#define GENE_H_4378354

#include<vector>
#include"vector.h"
#include"braininfo.h"


class GeneClass {
private:
  vector<long> DNA;

public:
  GeneClass();
  GeneClass(GeneClass &,GeneClass &);
  ~GeneClass();
  VectorClass Size();
  BrainInfoClass Brain();
  bool IsViable();
  bool MakeViable();
  double Color();
  double MaxSpeed();
  double GetEnergyPcnt();
  double Metabolism();
  long Reach();
  double GetStrength();
  double Variance(GeneClass &);
  long Lifespan();
  double MutationRate();
  double LC();
};

#endif
